# -*- coding: utf8 -*-
#
# cacaeditor    Libcaca ascii editor.
#
# Copyright (c) 2011 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This program is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What The Fuck You Want
# To Public License, Version 2, as published by Sam Hocevar. See
# http://sam.zoy.org/wtfpl/COPYING for more details.
#

"""
Libcaca ASCii Editor
"""

__version__ = "0.1.1"
__date__ = "$Date: 2012-01-03 00:02:45 +0100 (mar. 03 janv. 2012) $"

#debug
#import pdb

#standard module
import sys

#third party module
import caca

#debug global values
DEBUG = False
DEBUG_MODULES = ['core', 'handles', 'commands', 'gui', 'widgets', 'helpers']

#
# Application exceptions
#

class AppError(Exception):
    """ Application exception.
    """
    pass

class FileError(Exception):
    """ File exception.
    """
    pass

#
# Messages
#

ABOUT_MSG = """\
CacaEditor v%s (Python v%s - Libcaca v%s)

cacaeditor is a python application to easily draw ascii ART,
and export document in several formats. It works with Libcaca
Python bindings.

-----------------------------------------------------------------

Copyright (c) 2011 Alex Foulon <alxf@lavabit.com>
                   All Rights Reserved

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.
""" % (
    __version__,
    ".".join([str(i) for i in sys.version_info[0:3]]),
    caca.get_version(),
)

HELP_MSG = """\
This the help command.

Type `list' to see all available commands.
"""

#
# Constants
#

INTERRUPT, CONTINUE = 1, 2

COLOR_NAMES = {
    'black': caca.COLOR_BLACK,
    'blue': caca.COLOR_BLUE,
    'brown': caca.COLOR_BROWN,
    'cyan': caca.COLOR_CYAN,
    'darkgray': caca.COLOR_DARKGRAY,
    'green': caca.COLOR_GREEN,
    'lightblue': caca.COLOR_LIGHTBLUE,
    'lightcyan': caca.COLOR_LIGHTCYAN,
    'lightgray': caca.COLOR_LIGHTGRAY,
    'lightgreen': caca.COLOR_LIGHTGREEN,
    'lightmagenta': caca.COLOR_LIGHTMAGENTA,
    'lightred': caca.COLOR_LIGHTRED,
    'magenta': caca.COLOR_MAGENTA,
    'red': caca.COLOR_RED,
    'white': caca.COLOR_WHITE,
    'yellow': caca.COLOR_YELLOW,
}

INFO_MSG = {
    'READY':         "New [CTRL+N] Open [CTRL+O] Quit [CTRL+Q]",
    'OPENED':        "Document opened",
    'IMPORT':        "Move [ARROWS] Validate [RETURN]",
    'CHOOSEIMPORT':  "Import document",
    'CHOOSEEXPORT':  "Export document",
    'CHOOSECOLOR1':  "Change colors",
    'CHOOSECOLOR2':  "Change colors",
    'DRAWTEXT':      "Insert text, Validate [ESCAPE]",
    'MOVETEXT':      "Move [ARROWS] Transform [CTRL+T] Validate [RETURN]",
    'TRANSFORM':     "Add a transform operation",
    'DRAWLINE':      "Insert line, Validate [RETURN]",
    'MOVELINE':      "Move [ARROWS] Validate [RETURN]",
    'DRAWBOX':       "Insert box, Validate [RETURN]",
    'MOVEBOX':       "Move [ARROWS] Validate [RETURN]",
    'DRAWTRIANGLE1': "Insert triangle, choose second corner. Validate [RETURN]",
    'DRAWTRIANGLE2': "Insert triangle, choose last corner. Validate [RETURN]",
    'MOVETRIANGLE':  "Move [ARROWS] Validate [RETURN]",
    'DRAWCIRCLE':    "Insert circle, Validate [RETURN]",
    'MOVECIRCLE':    "Move [ARROWS] Validate [RETURN]",
    'DRAWELLIPSE1':  "Insert ellipse, choose first radius. Validate [RETURN]",
    'DRAWELLIPSE2':  "Insert ellipse, choose second radius. Validate [RETURN]",
    'MOVEELLIPSE':   "Move [ARROWS] Validate [RETURN]",
}

TRANSFORM_OPERATIONS = [
    "flip",
    "flop",
    "rotate_180",
    "rotate_left",
    "rotate_right",
    "normal"
]

#Each key is a menu, and each tuple is an entry for the menu
#that contains data for submenu item.
#(tag, label, keybinding, callback)
# - tag must be unique id string to be used as method name
#   like: self.action_open_document(). In this example the tag is
#   "open_document".
# - a tuple with None for each value is a menu separator.
# - to disable a keybinding, put a None value.
MENU_DATA = {
    '_index': [
        "File",
        "Draw",
        "Help"
    ],
    'File': [
        ("new_document", "New Document", "ctrl+n"),
        ("open_document", "Open Document", "ctrl+o"),
        ("close_document", "Close Document", "ctrl+x"),
        (None, None, None),
        ("save_document", "Save Document", 's'),
        ("save_document_as", "Save Document as", 'S'),
        (None, None, None),
        ("export_document", "Export Document", "ctrl+e"),
        ("import_document", "Import Document", "ctrl+j"),
        (None, None, None),
        ("quit_application", "Quit Application", "ctrl+q"),
    ],
    'Draw': [
        ("insert_text", "Insert Text", "i"),
        (None, None, None),
        ("insert_line", "Insert Line", "L"),
        ("insert_thin_line", "Insert Thin Line", "l"),
        ("insert_triangle", "Insert Triangle", "T"),
        ("insert_thin_triangle", "Insert Thin Triangle", 't'),
        ("insert_box", "Insert Box", "B"),
        ("insert_thin_box", "Insert Thin Box", "b"),
        ("insert_circle", "Insert Circle", "O"),
        ("insert_ellipse", "Insert Ellipse", "Z"),
        ("insert_thin_ellipse", "Insert Thin Ellipse", "z"),
        (None, None, None),
        ("fill_triangle", "Fill Triangle", None),
        ("fill_box", "Fill Box", None),
        ("fill_ellipse", "Fill Ellipse", None),
        (None, None, None),
        ("change_colors", "Change Colors", "c"),
        ("reset_colors", "Reset Colors", "C"),
    ],
    'Help': [
        ("list_format", "List formats", "ctrl+f"),
        (None, None, None),
        ("about", "About CacaEditor", "ctrl+a"),
    ],
}

#
# Keybindings lists shortcuts
#

KEYS_ABORT = [caca.KEY_CTRL_C]

KEYS_VALIDATE = [caca.KEY_ESCAPE]

KEYS_ARROW = [caca.KEY_LEFT, caca.KEY_RIGHT, caca.KEY_UP, caca.KEY_DOWN]

KEYS_YES = [ord('y'), ord('Y')]

KEYS_ASCII = range(32, 127)

KEYS_ALPHA = range(65, 91) + range(97, 123)

KEYS_NUM = range(48, 58)

KEYS_ALPHANUM = KEYS_ALPHA + KEYS_NUM

